<?php

namespace Drupal\dual_path_image_save\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\Core\Form\FormStateInterface;

/**
 * Field handler to show the image from the custom path.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("dual_path_image")
 */
class DualPathImage extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['image_field'] = ['default' => ''];
    $options['image_style'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $fields_with_custom_path = [];
    
    // Load all image field configurations for nodes.
    $field_configs = \Drupal::entityTypeManager()->getStorage('field_config')->loadByProperties(['entity_type' => 'node', 'field_type' => 'image']);
//\Drupal::messenger()->addMessage(print_r($field_configs, TRUE));

    
    foreach ($field_configs as $field_config) {
      //$settings = $field_config->getSettings();
      $settings = $field_config->getThirdPartySettings('dual_path_image_save');

      if (isset($settings['path']) && !empty($settings['path'])) {
        //$fields_with_custom_path[$field_config->getName()] = $field_config->getLabel();
        $fields_with_custom_path[$field_config->getName()] = $field_config->getLabel() . ' (' . $field_config->getName() . ')';

      }
    }

    $form['image_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Image Field'),
      '#description' => $this->t('Select the image field you want to display from the custom path.'),
      '#options' => $fields_with_custom_path,
      '#default_value' => $this->options['image_field'],
    ];

    // Load image styles
    $styles = image_style_options(FALSE);
    $form['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image Style'),
      '#description' => $this->t('Choose the image style you want to apply to the image.'),
      '#options' => $styles,
      '#default_value' => $this->options['image_style'],
    ];

    parent::buildOptionsForm($form, $form_state);
  }
    
  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $field_name = $this->options['image_field'];
    $original_uri = $values->_entity->{$field_name}->entity->uri->value;

    // Dynamically determine content type.
    $bundle = $values->_entity->bundle();

    // Extract custom path from field settings.
    $field_config = \Drupal\field\Entity\FieldConfig::loadByName('node', $bundle, $field_name);
    $third_party_settings = $field_config->getThirdPartySettings('dual_path_image_save');
    $custom_path = isset($third_party_settings['path']) ? $third_party_settings['path'] : 'public://';


    $filename = \Drupal::service('file_system')->basename($original_uri);
    $custom_uri = $custom_path . '/' . $filename;

    $style_name = $this->options['image_style'];
    if (!empty($style_name) && $style = \Drupal::entityTypeManager()->getStorage('image_style')->load($style_name)) {
        $custom_uri = $style->buildUrl($custom_uri);
    }

    return [
      '#theme' => 'image',
      '#uri' => $custom_uri,
      '#alt' => $values->_entity->{$field_name}->alt, // Dynamically retrieve the alt text
    ];
  }

    /**
     * {@inheritdoc}
     */
    public function query() {
      // We don't query the database for this field.
      // Everything is handled in the render() method.
      return;
    }

}


